import argparse
import os
from .uploader import upload_file, upload_xml_files_from_folder
from rich import print
from .check_version import should_check_version, check_latest_version
from .__init__ import __version__

def run():
	parser = argparse.ArgumentParser(description="Upload a single .xml file or all .xml files in a folder to an API endpoint.")
	parser.add_argument('action', choices=['UPLOAD_RESULTS'], help='Action to do')
	parser.add_argument('path', help='Path to .xml file or folder containing .xml files')
	parser.add_argument('--api_key', required=True, help='Bugasura API Key')
	parser.add_argument('--team_id', required=True, help='Bugasura Team Id')
	parser.add_argument('--project_id', required=True, help='Bugasura Project Id')
	parser.add_argument('--testrun_id', required=False, help='Bugasura Testrun Id. If not passed, new Testrun will be created in the Bugasura Project')
	parser.add_argument('--server', required=False, choices=['local', 'live', 'stage', 'facilio', 'shoppersstop', 'frammer', 'jupiter', 'trustrace', 'emotive', 'testpert'], help='Enterprise Server Names')
	parser.add_argument('--version', '-v', action='version', version=__version__)
	args = parser.parse_args()
	api_base_url = "http://localhost/api.appachhi.com/"
	is_version_check_done = False

	if args.server:
		if args.server.lower() == "local":
			api_base_url = "http://localhost/api.appachhi.com/"
		elif args.server.lower() == "stage":
			api_base_url = "https://api.stage.bugasura.io/"
		elif args.server.lower() == "live":
			api_base_url = "https://api.bugasura.io/"
		elif args.server.lower() == "facilio":
			api_base_url = "https://api.facilio.bugasura.io/"
		elif args.server.lower() == "shoppersstop":
			api_base_url = "https://api.shoppersstop.bugasura.io/"
		elif args.server.lower() == "frammer":
			api_base_url = "https://api.frammer.bugasura.io/"
		elif args.server.lower() == "jupiter":
			api_base_url = "https://api.jupiter.bugasura.io/"
		elif args.server.lower() == "trustrace":
			api_base_url = "https://api.trustrace.bugasura.io/"
		elif args.server.lower() == "emotive":
			api_base_url = "https://api.emotive.bugasura.io/"
		elif args.server.lower() == "testpert":
			api_base_url = "https://api.testpert.bugasura.io/"

	if not args.testrun_id:
		args.testrun_id = ''

	if not args.action:
		args.action = 'UPLOAD_RESULTS'

	try:
		if os.path.isfile(args.path):
			result = upload_file(args.path, args.api_key, args.team_id, args.project_id, args.testrun_id, api_base_url)
			report_link = result.get('apiResponse', {}).get('reportLink')
			if report_link:
				print(f"[bold green]Upload successful![/bold green] [bold magenta]🔗 Bugasura Test Run Link: [/bold magenta] [link={report_link}]{report_link}[/link]")
			else:
				print("[bold green]Upload successful![/bold green]")
		elif os.path.isdir(args.path):
			result = upload_xml_files_from_folder(args.path, args.api_key, args.team_id, args.project_id, args.testrun_id, api_base_url)
			report_link = result.get('apiResponse', {}).get('reportLink')
			if report_link:
				print(f"[bold green]Upload successful![/bold green] [bold magenta]🔗 Bugasura Test Run Link: [/bold magenta] [link={report_link}]{report_link}[/link]")
			else:
				print("[bold green]Upload successful![/bold green]")
		else:
			raise ValueError(f"[bold red]Invalid path: {args.path}[/bold red]")
	except Exception as e:
		print(f"[bold red]❌ Upload failed: {str(e)}[/bold red]")
		check_latest_version()
		is_version_check_done = True

	if should_check_version() and (is_version_check_done == False):
		check_latest_version()


import os
import glob
import requests
from typing import List
from .config import Config

def upload_file(filepath: str, api_key: str, team_id: str, project_id: str, testrun_id: str, api_base_url: str) -> List[dict]:
	if not os.path.isfile(filepath):
		raise FileNotFoundError(f"File not found: {filepath}")
	if not filepath.lower().endswith(tuple(Config().allowed_extensions)):
		raise ValueError(f"Only '{tuple(Config().allowed_extensions)}' files are allowed")

	http_method = 'FILE'  # 'POST' or 'GET'
	api_endpoint = 'tbreports/uploadAutomationRunResults'

	if not api_key:
		api_key = Config().default_api_key

	params_dict = { 'teamId': team_id, 'appId': project_id, 'reportId': testrun_id, 'isCLI': 1 }

	files = {'attachments': open(filepath, 'rb')}

	# Call the curl_request method to upload response
	response =  curl_request(api_base_url, http_method, api_endpoint, api_key, params_dict, files)

	# Error Check
	if response['status'] != 'OK' or response['apiResponse']['status'] != 'OK':
		return response

	return response

def upload_xml_files_from_folder(folder: str, api_key: str, team_id: str, project_id: str, testrun_id: str, api_base_url: str) -> List[dict]:
	filepaths = glob.glob(os.path.join(folder, tuple(Config().allowed_extensions)))
	if not filepaths:
		raise FileNotFoundError(f"No '{tuple(Config().allowed_extensions)}' files found in folder '{folder}'")

	http_method = 'FILE'
	api_endpoint = 'tbreports/uploadAutomationRunResults'

	if not api_key:
		api_key = Config().default_api_key

	params_dict = { 'teamId': team_id, 'appId': project_id, 'reportId': testrun_id }

	# Build the 'attachments' file list for all xml files
	files = [('attachments', open(fp, 'rb')) for fp in filepaths]

	# Call the curl_request method to upload respons
	response = curl_request(api_base_url, http_method, api_endpoint, api_key, params_dict, files)

	if response['status'] != 'OK' or response['apiResponse']['status'] != 'OK':
		return response

	return response

def curl_request(api_base_url="http://localhost/api.appachhi.com/", http_method='POST', api_endpoint='', api_key='', params_dict=None, files=None):
	response = _initialize_response()

	http_method, response = _validate_params(http_method, api_endpoint, api_key, params_dict, response)
	headers = _build_headers(api_key, http_method)

	try:
		api_url = _build_api_url(api_base_url, api_endpoint, http_method, params_dict)
		response = _make_request(http_method, api_url, headers, params_dict, response, files)
	except requests.RequestException as e:
		raise ValueError(f"ERROR: {str(e)}")
		response['message'] = str(e)
		return response

	return response

def _initialize_response():
	return {
		'status': 'ERROR',
		'message': ''
	}

def _validate_params(http_method, api_endpoint, api_key, params_dict, response):
	http_method = http_method.upper()
	if http_method not in ['GET', 'POST', 'FILE']:
		raise ValueError(f"httpMethod=[{http_method}] is not valid/supported")
		response['message'] = f"httpMethod=[{http_method}] is not valid/supported"
		return http_method, response

	if not api_endpoint:
		raise ValueError(f"apiEndpoint=[{api_endpoint}] is missing")
		response['message'] = f"apiEndpoint=[{api_endpoint}] is missing"
		return http_method, response

	if not api_key:
		raise ValueError(f"INFO: apiKey=[{api_key}] is missing. Default api Key has been set")
		response['message'] = "api key is missing"
		return http_method, response

	if params_dict is None or not isinstance(params_dict, dict):
		raise ValueError(f"Parameters not set correctly to make request")
		params_dict = {}

	return http_method, response

def _build_headers(api_key, http_method):

	expect_header = {"Expect": "100-continue"} if http_method == 'POST' else {}
	headers = {
		'Accept': 'application/json',
		'Authorization': f'Basic {api_key}'
	}

	# Merge the Expect header if it exists
	headers.update(expect_header)

	return headers

def _build_api_url(api_base_url, api_endpoint, http_method, params_dict):
	if http_method == 'GET' and params_dict:
		query_string = '&'.join(f"{key}={value}" for key, value in params_dict.items())
		api_endpoint += f"?{query_string}"
	return api_base_url + api_endpoint

def _make_request(http_method, api_url, headers, params_dict, response, file):
	if http_method == 'POST':
		resp = requests.post(api_url, headers=headers, data=params_dict)
	elif http_method == 'FILE':
		resp = requests.post(api_url, headers=headers, files=file, data=params_dict)
	else:
		resp = requests.get(api_url, headers=headers)

	if resp.status_code != 200:
		raise ValueError(f"Request ERROR: {resp.text}")
		response['message'] = resp.text
		return response

	try:
		api_response = resp.json()
	except ValueError:
		raise ValueError(f"Response could not be parsed")
		response['message'] = 'Could not parse JSON response'
		return response

	if not isinstance(api_response, dict):
		raise ValueError(f"Could not fetch details from the API.")
		response['message'] = 'Could not fetch details from the API'
		response['apiResponse'] = api_response
		return response

	response['status'] = 'OK'
	response['message'] = 'Successfully fetched the API array.'
	response['apiResponse'] = api_response

	if api_response.get('status') != 'OK':
		raise ValueError(f"ERROR: {api_response.get('message')}")
		return response

	return response


# DEFAULT_API_URL = "https://api.example.com/upload"
# ALLOWED_EXTENSIONS = [".xml"]
# LOGGING_ENABLED = True
# VERSION = "0.1.0"

import os

class Config:
	def __init__(self):
		# self.server = "LOCAL"
		# self.api_base_url = "http://localhost/api.appachhi.com/"
		self.allowed_extensions = [".xml"]
		self.default_api_key = "0c98159b0990e30f6af82a08a3c0c48077bd6545"
		# self.setup_configuration()

	# def setup_configuration(self):
	# 	if self.server == "LOCAL":
	# 		self.api_base_url = "http://localhost/api.appachhi.com/"
	# 	elif self.server == "STAGE":
	# 		self.api_base_url = "https://api.stage.bugasura.io/"
	# 	elif self.server == "LIVE":
	# 		self.api_base_url = "https://api.bugasura.io/"
	# 	elif self.server == "CUSTOM":
	# 		self.api_base_url = ""
	# 	else:
	# 		self.api_base_url = "https://api.stage.bugasura.io/"



import requests
import sys
import os
import time
from packaging import version
from rich import print

try:
	from importlib.metadata import version as get_installed_version
except ImportError:
	from importlib_metadata import version as get_installed_version  # type: ignore [python < 3.8]

PACKAGE_NAME = "bguploaderlocal"
CACHE_DIR = os.path.join(os.path.expanduser('~'), '.cache', PACKAGE_NAME)
CACHE_FILE = os.path.join(CACHE_DIR, 'last_version_check')
CHECK_INTERVAL = 86400  # 24 hours in seconds

def should_check_version():
	try:
		# Create cache dir if it doesn't exist
		os.makedirs(CACHE_DIR, exist_ok=True)
		# Read last check time
		if os.path.exists(CACHE_FILE):
			with open(CACHE_FILE, 'rt') as f:
				last_check = float(f.read())
				if time.time() - last_check < CHECK_INTERVAL:
					return False
		# Write current time
		with open(CACHE_FILE, 'wt') as f:
			f.write(str(time.time()))
	except Exception:
		pass
	return True

def check_latest_version():
	try:
		current_version = get_installed_version(PACKAGE_NAME)
		pypi_url = f"https://pypi.org/pypi/{PACKAGE_NAME}/json"
		resp = requests.get(pypi_url, timeout=1.5)
		if resp.status_code == 200:
			latest_version = resp.json()["info"]["version"]
			if version.parse(latest_version) > version.parse(current_version):
				print(f"\n\033[93mNOTICE\033[0m: You are using '{PACKAGE_NAME}' version {current_version}; however, version {latest_version} is available.")
				print(f"Upgrade with: [bold green]pip install --upgrade {PACKAGE_NAME}[/bold green]\n")
	except Exception:
		pass


from setuptools import setup, find_packages

setup(
	name='bguploaderlocal',
	version='0.0.20',
	packages=find_packages(),
	install_requires=['requests','urllib3<2','rich'],
	entry_points={
		'console_scripts': [
			'bguploaderlocal=bguploaderlocal.cli:run',
		],
	},
	author='pathik-appachhi',
	description='CLI to upload JUnit .xml files.',
	python_requires='>=3.6',
)